#!/bin/bash

# EarlyMon May 2013
# Team Solar Federation
# Team Leader: argedion
# Special thanks to other team members -
#   BabyBlues
#   Scary Alien
#   Out Of Phases
#   palmtree5

# Suggested use with Mini-SDK from
#  http://androidforums.com/faqs/443072-adb-guide-updated-12-05-2011-a.html
#  Installer _must_ have been run as: sh sdk.sh

# credit for FILTER method to find cards goes to -
# http://renzhi.ca/2012/02/03/how-to-list-all-sd-cards-on-android/ 

# Root not required with MTP, works for many things, not all.

# Install alongside of adb. Saves locally if using cygwin or Mini-SDK,
# under HOME directory otherwise.

platform=`uname -s`
here=`pwd`
if [[ "$platform" =~ "CYGWIN"* || "$here" =~ "$HOME"* ]] ; then
  save_path="./android_storage_backups"
else
  save_path=${HOME}"/android_storage_backups"
fi
save_path=`echo $save_path | sed 's/ /\\\ /g'`

ADB=`which adb`
if [ -z "$ADB" ] ; then
  ADB="./adb"
fi

echo " "
echo " "
echo "Storage Backup from Team Solar Federation...."
echo "For best results reboot your device before proceeding."
echo "The reboot is recommended but not required."
echo "Wait a minute or two after rebooting to ensure that all file systems are ready."
echo " "
read -p "Press [Enter] key to start backup or control-c to quit..."

$ADB start-server
snum=`$ADB get-serialno | tr -d '\r\n\t /\\\' | sed '/^$/d'`

# Find all cards

MPOINT="sed 's+^.* /+/+' | sed 's/ .*$//'"
FILTER="egrep 'vfat|/mnt' | grep /dev/block/vold | grep -v /mnt/asec | \
         grep -v /mnt/secure"

icard=`$ADB shell "mount" | eval $FILTER | eval $MPOINT`
ecard=`$ADB shell "mount" | grep /dev/fuse | eval $MPOINT`

CARDS="((echo $icard | tr ' ' '\n'); echo $ecard) | sed '/^$/d' | sort -u -i"

# Get number of sd cards

nc=`eval $CARDS | wc -l`

# Get the name of the card that must be there, the optional second one,
# and bail out if nothing or too many found this way.

if [[ "$nc" =~ 2 ]] ; then
  card2=`eval $CARDS | tail -1`
elif [[ "$nc" =~ 0 ]] ; then
  echo "Lost in space... can't find sdcard or equivalent, bailing out! "
  exit
elif [[ ! "$nc" =~ 1 ]] ; then
  echo "Lost in space... more than 2 sdcards or equivalent, bailing out! "
  exit
fi
card1=`eval $CARDS | head -1`

# Get target save directory

target1=${save_path}"/${snum}_"`date +"%Y%m%d"`
if [ -d "$target1" ] ; then
  target1=${target1}`date +"%H"`
  if [ -d "$target1" ] ; then
    target1=${target1}`date +"%M"`
    if [ -d "$target1" ] ; then
      target1=${target1}`date +"%S"`
    fi
  fi
fi

# Backup primary and secondary if mounted together

mkdir -p $target1

$ADB pull $card1 $target1

# LOST.DIR isn't necessary, never restored by user

cd $target1
rm -Rf LOST.DIR
cd ..

# Backup secondary if not mounted with primary

if [[ "$nc" =~ 2 &&  "$card2" != "$card1"* ]] ; then
  target2=${target1}"_2nd"
  mkdir -p $target2
  $ADB pull $card2 $target2
  cd $target2
  rm -Rf LOST.DIR
  cd ..
fi
